#ifndef CON_AWF1MATC_H
#define CON_AWF1MATC_H

#include "genfile.h"

/**
   This class defines model for water flow in soils.


   Order of the Cmu->nonmechtransq array components
   id    | description
   ------+---------------------------------------------------------------------------
   
    transport:
    
    0    | alpha*s*pw ... part for effective stress computing
    1    | d_eps_p  ... increment of plastic strains (caused by grains rearrangement)
    2    | eps_p    ... total plastic strains (caused by grains rearrangement)
    3    | sat      ... degree of saturation
    4    | s        ... suction stress

    mechanical:

    5    | p_c      ... preconsolidation pressure
    6    | sigma_m_eff  ... effective mean stress (i1s)
    7    | e_lambda1 ...
    8    | e_ini     ...

*/

class con_awf1matc
{
 public:
  con_awf1matc();    //constructor
  ~con_awf1matc();   //destructor

  void read(XFILE *in);

  void rhs_volume (matrix &d,long ri,long ci,long ipp);
  void rhs1d1 (matrix &d,long ri,long ci,long ipp);
  void rhs2d1 (matrix &d,long ri,long ci,long ipp);
  void rhs3d1 (matrix &d,long ri,long ci,long ipp);

  double get_sw(double pw);
  double get_dsw_dpw(double pw);
  double get_krw(double pw);
  double get_phi();
  double get_kintr();
  double get_rhos();
  double get_alpha();
  double get_emod();
  double get_nu();
  double get_ks();
  double get_kw();
  double get_muw();

  double get_kuw(double pw);
  double get_kwu(double pw);
  double get_kww(double pw,long ipp);

  double get_capuw(double pw);
  double get_capwu(double pw);
  double get_capww(double pw);

  double get_fw1(double pw);
  double get_fu1(double pw);
  void updateval (long ipp,long ido);
  void initval(long ipp, long ido);

  double get_eigstr(double pw);  

  double get_suction(double pw);

  double nu;

 private:
  
  long model_type;
  double ab,bb;
  double emod,alpha,ks,phi0,rhos;
  double kw,rhow,muw0;
  double kintr;
  double ssat,sirr,alfa,ksat,gamaw;
  double hp_min,htz,r;
  double delta,expn;
  double k0,e0,lam0,lam,m,sigma_m_bar;
  double kappa,sigma_m_eff_bar;
};  

#endif
