#include "globmatc.h"
#include "globmat.h"
#include "globmatt.h"
#include "global.h"
#include "globalt.h"
#include "globalc.h"
#include "loadcase.h"
#include "element.h"
#include "elemswitch.h"
#include "elemswitcht.h"
#include "elemswitchc.h"

/**
   function assembles %matrix which contains stiffness and conductivity matrices
   
   @param lcid - load case id

   JK
*/
void zero_order_matrix (long lcid)
{
  long i,mndofe,tndofe;
  ivector rcn,ccn;
  matrix lm;
  
  if (D0mat==NULL)  D0mat = new gmatrix;
    
  //  provizorni gtopologie (kombinace Gtm a Gtt)
  
  //D0mat->ts=Cp->tstord0;
  D0mat->setval (Cp->ssle);
  D0mat->initiate (Gtu,Ndofc,Cp->tstord0,Mesprc,Outc);

  
  //  mazani provizorni topologie
    
  //D0mat->setval (Cp->ssle.tlinsol,Cp->ssle.prec.pt,Cp->ssle.ni,Cp->ssle.err);
  
  //fprintf (Outc,"\n\n\n na zacatku\nMATICE VODIVOSTI (TUHOSTI) \n\n\n");
  //D0mat->printmat (Outc);
  
  //***************************************
  //  contributions from stiffness matrices
  //***************************************
  //  contributions from MEFEL part
  for (i=0;i<Mt->ne;i++){
    mndofe = Gtm->give_ndofe (i);
    
    allocm (mndofe,mndofe,lm);
    stiffmat (i,lcid,lm);
    
    allocv (mndofe,rcn);
    //rcn = new long [mndofe];
    Gtm->give_code_numbers (i,rcn.a);
    
    D0mat->localize (lm,rcn,i);
    
    destrm (lm);
    destrv (rcn);
    //delete [] rcn;
  }
  

  //fprintf (Outc,"\n\n\n mefel\nMATICE TUHOSTI \n\n\n");
  //D0mat->printmat (Outc);

  //******************************************
  //  contributions from conductivity matrices
  //******************************************
  //  contributions from TRFEL part
  for (i=0;i<Tt->ne;i++){
    tndofe = Gtt->give_ndofe (i);
    
    allocm (tndofe,tndofe,lm);
    conductmat (i,lcid,lm);
    
    allocv (tndofe,rcn);
    //rcn = new long [tndofe];
    Gtt->give_code_numbers (i,rcn.a);
    
    D0mat->localize (lm,rcn,i);
    
    destrm (lm);
    destrv (rcn);
    //delete [] rcn;
  }
    
  //fprintf (Outc,"\n\n\n trfel\nMATICE VODIVOSTI \n\n\n");
  //D0mat->printmat (Outc);

  //  contributions from METR upper part
  for (i=0;i<Ct->ne;i++){
    mndofe = Gtm->give_ndofe (i);
    tndofe = Gtt->give_ndofe (i);
    
    allocm (mndofe,tndofe,lm);
    upper_cond_coupl_mat (i,lcid,lm);
    
    allocv (mndofe,rcn);
    //rcn = new long [mndofe];
    allocv (tndofe,ccn);
    //ccn = new long [tndofe];
    Gtm->give_code_numbers (i,rcn.a);
    Gtt->give_code_numbers (i,ccn.a);
    
    D0mat->glocalize (lm,rcn,ccn);
    
    destrm (lm);
    destrv (rcn);
    //delete [] rcn;
    destrv (ccn);
    //delete [] ccn;
  }
    
  //fprintf (Outc,"\n\n\n metru\nMATICE VODIVOSTI-TUHOSTI \n\n\n");
  //D0mat->printmat (Outc);
			  
  //  contributions from METR lower part
  for (i=0;i<Ct->ne;i++){
    mndofe = Gtm->give_ndofe (i);
    tndofe = Gtt->give_ndofe (i);
    
    allocm (tndofe,mndofe,lm);
    lower_cond_coupl_mat (i,lcid,lm);
    
    allocv (tndofe,rcn);
    //rcn = new long [tndofe];
    allocv (mndofe,ccn);
    //ccn = new long [mndofe];
    Gtm->give_code_numbers (i,ccn.a);
    Gtt->give_code_numbers (i,rcn.a);
    
    D0mat->glocalize (lm,rcn,ccn);
    
    destrm (lm);
    destrv (rcn);
    //delete [] rcn;
    destrv (ccn);
    //delete [] ccn;
  }
  
  //fprintf (Outc,"\n\n\n metrl\nMATICE VODIVOSTI-TUHOSTI \n\n\n");
  //D0mat->printmat (Outc);
  
  //D0mat->prepmat (Gtu,Cp->zero,Mesprc);
  D0mat->prepmat (Cp->zero,Mesprc);
}

/**
   function assembles general capacity %matrix
   
   @param lcid - load case id

   JK
*/
void first_order_matrix (long lcid)
{
  long i,mndofe,tndofe;
  ivector rcn,ccn;
  matrix lm;

  if (D1mat==NULL)  D1mat = new gmatrix;

  //  sdruzena topologie
  //D1mat->ts=Cp->tstord1;
  D1mat->setval (Cp->ssle);
  D1mat->initiate (Gtu,Ndofc,Cp->tstord1,Mesprc,Outc);

  
  //D1mat->setval (Cp->ssle.tlinsol,Cp->ssle.prec.pt,Cp->ssle.ni,Cp->ssle.err);


  //  contributions from MEFEL part
  /* for (i=0;i<Mt->ne;i++){//pokus!!??
     mndofe = Gtm->give_ndofe (i);
     
     allocm (mndofe,mndofe,lm);
     stiffmat (i,lcid,lm);
     
     rcn = new long [mndofe];
     Gtm->give_code_numbers (i,rcn);
     
     D1mat->localize (lm,rcn,i);
     
     destrm (lm);
     delete [] rcn;
     }
  */
  
  //  contributions from TRFEL part
  for (i=0;i<Tt->ne;i++){
    tndofe = Gtt->give_ndofe (i);
    
    allocm (tndofe,tndofe,lm);
    capacmat (i,lcid,lm);
    
    allocv (tndofe,rcn);
    //rcn = new long [tndofe];
    Gtt->give_code_numbers (i,rcn.a);
    
    D1mat->localize (lm,rcn,i);

    destrm (lm);
    destrv (rcn);
    //delete [] rcn;
  }



  //  contributions from METR upper part
  for (i=0;i<Ct->ne;i++){
    mndofe = Gtm->give_ndofe (i);
    tndofe = Gtt->give_ndofe (i);
    
    allocm (mndofe,tndofe,lm);
    upper_cap_coupl_mat (i,lcid,lm);
    
    allocv (mndofe,rcn);
    //rcn = new long [mndofe];
    allocv (tndofe,ccn);
    //ccn = new long [tndofe];
    Gtm->give_code_numbers (i,rcn.a);
    Gtt->give_code_numbers (i,ccn.a);
    
    D1mat->glocalize (lm,rcn,ccn);
    
    destrm (lm);
    destrv (rcn);
    //delete [] rcn;
    destrv (ccn);
    //delete [] ccn;
  }


  //  contributions from METR lower part
  for (i=0;i<Ct->ne;i++){
    mndofe = Gtm->give_ndofe (i);
    tndofe = Gtt->give_ndofe (i);
    
    allocm (tndofe,mndofe,lm);
    lower_cap_coupl_mat (i,lcid,lm);
    
    allocv (tndofe,rcn);
    //rcn = new long [tndofe];
    allocv (mndofe,ccn);
    //ccn = new long [mndofe];
    Gtm->give_code_numbers (i,ccn.a);
    Gtt->give_code_numbers (i,rcn.a);
    
    D1mat->glocalize (lm,rcn,ccn);
    
    destrm (lm);
    destrv (rcn);
    //delete [] rcn;
    destrv (ccn);
    //delete [] ccn;
  }
  


  //D1mat->prepmat (Gtu,Cp->zero,Mesprc);
  D1mat->prepmat (Cp->zero,Mesprc);
}


/**
   function computes %vector of generalized internal forces
   general internal force is notation of internal force or internal flux
   
   @param lcid   - load case id
   @param intfor - internal forces and fluxes

   JK, 28.7.2001
*/
void internal_gforces (long lcid,double *intfor)
{
  double *ifort,*iform;

  ifort= new double [Ndofc];
  nullv (ifort,Ndofc);
  iform= new double [Ndofc];
  nullv (iform,Ndofc);
  nullv (intfor,Ndofc);

  //  actual nodal forces MEFEL
  internal_forces (lcid,iform);
  addv (intfor,iform,Ndofc);

  //  actual nodal fluxes TRFEL
  internal_fluxes (ifort,Ndofc);
  addv (intfor,ifort,Ndofc);

  // actual nodal forces  and fluxes METR
  //internal_coup_fluxes (av,Ndofc);
  //internal_coup_forces (lcid,intfor);
  
  delete [] iform;
  delete [] ifort;
}


/**
   function computes %vector of increments of generalized internal forces
   general internal force is notation of internal force or internal flux
   
   @param lcid   - load case id
   @param intfor - increments of internal forces and fluxes

   12/9/2008, TKr
*/
void incr_internal_gforces (long lcid,double *intfor)
{
  double *ifort,*iform;

  ifort= new double [Ndofc];
  nullv (ifort,Ndofc);
  iform= new double [Ndofc];
  nullv (iform,Ndofc);
  nullv (intfor,Ndofc);
  
  //  actual increments of nodal forces MEFEL
  incr_internal_forces (lcid,iform);
  addv (intfor,iform,Ndofc);
  
  //  actual increments of nodal fluxes TRFEL
  //incr_internal_fluxes (ifort,Ndofc);
  //addv (intfor,ifort,Ndofc);

  // actual increments of nodal forces  and fluxes METR
  //incr_internal_coup_fluxes (av,Ndofc);
  //incr_internal_coup_forces (lcid,intfor);
  
  delete [] iform;
  delete [] ifort;
}


/**
   function extracts TRFEL values on one element

   @param lcid - number of load case
   @param r - vector of nodal values
   @param cn - array containing code numbers
   @param ndofe - number of DOFs on actual element

   JK, 9.7.2001
*/
void elemvaluesc (long lcid,double *r,long *cn,long ndofe)
{
  long i,ii;

  for (i=0;i<ndofe;i++){
    ii=cn[i];
    if (ii<0)   r[i]=Tb->lc[lcid].pv[0-ii-1].getval();
    if (ii==0)  r[i]=0.0;
    if (ii>0)   r[i]=Lsrst->lhsi[ii-1]+Lsrst->lhs[ii-1];
  }
}



/**
   function computes %vector of generalized right hand side
   
   @param lcid - load case id
   @param rhs - array of right hand side
   @param flv - array of load %vector caused by forces only (default value is NULL)
   
   @return The function returns assembled vectors of righ hand side and load vector caused by force.

   JK, 28.7.2001,
   Modified by Tomas Krejci 18/05/2012
*/
void metr_right_hand_side (long lcid,double *rhs,double *flv)
{
  double *av;

  av = new double [Ndofc];

  nullv (rhs,Ndofc);
  nullv (flv,Ndofc);
  nullv (av,Ndofc);
  
  null_mefel_tempstrains();
  null_metr_tempstrains();

  mefel_right_hand_side (lcid,rhs,flv);//MEFEL part
  trfel_right_hand_side (lcid,av,Ndofc);//TRFEL part
  addv (rhs,av,Ndofc);
  
  nullv (av,Ndofc);
  right_hand_side (lcid,av,Ndofc);//METR part
  addv (rhs,av,Ndofc);

  delete [] av;
}


/**
   function updates material data on element int. points
   
   14/05/2010, TKr
*/
void updateval()
{
  //updating of int. points material models
  Tm->updateipval ();
  Mm->updateipval();
  Cmu->updateipval();
  //Cml->updateipval();
}


/**
   function copies coupling data
   
   14/05/2010, TKr
*/
void copy_data()
{
  if (Cp->tprob==fully_coupled_mech_trans){
    //computation of strains for corect effective stresses
    //computation of strains due to temperature effect
    null_mefel_tempstrains();
    null_metr_tempstrains();
    compute_metr_tempstrains();
    copy_temprstrains_metr_mefel();
    
    //eigenstrains (nonmechanical strains):
    null_mefel_eigstrains();
    null_metr_eigstrains();
    compute_metr_eigstrains();
    copy_eigstrains_metr_mefel();
    
    //  copying of METR data to MEFEL
    copy_nonmechtransq_metr_mefel();
    //mefel_metr ();
    //  copying of transport data to METR
    //trfel_metr ();
    //  copying of coupling data to MEFEL
    //metr_mefel ();
    //  copying of coupling data to TRFEL
    //metr_trfel ();
  }
  
  if ((Cp->tprob==par_coupl_mech_trans) || (Cp->tprob==growing_par_coupl_mech_trans)){   
    //  computing transport data
    compute_trfel_eigstrains();
    //  copying of transport data to MEFEL
    trfel_mefel ();
    //  copying of mechanical data to TRFEL
    mefel_trfel ();
  }
}

/**
   function approximates data on element int. points
   
   JK, 28.7.2001
*/
void approximationc ()
{
  //  approximation of nodal values into integration points of TRFEL
  approximation ();  
  
  //  approximation of nodal values into integration points of METR
  approximationcoup ();
}





/**
   function approximates data on element int. points on coupling mesh
   
   JK, 28.7.2001
*/
void approximationcoup ()
{
  long i;
  
  switch (Cp->tmatt){
  case mech_onemedium:{
    for (i=0;i<Ct->ne;i++){
      intpointvaluesc (i);
      intpointstrainsc (i);
      intpointgradientsc (i);
    }
    break;
  }
  case mech_twomedia:{
    for (i=0;i<Ct->ne;i++){
      intpointvaluesc (i);
      intpointstrainsc (i);
      intpointgradientsc (i);
    }
    break;
  }
  case mech_threemedia:{
    for (i=0;i<Ct->ne;i++){
      intpointvaluesc (i);
      intpointstrainsc (i);
      intpointgradientsc (i);
    }
    break;
  }
  default:{
    fprintf (stderr,"\n\n unhknown transported matter is required in function");
    fprintf (stderr,"\n approximationcoup (file %s, line %d).\n",__FILE__,__LINE__);
  }
  }
  
}


/**
   function assembles %vector of coupling nodal internal fluxes and forces

   @param lcid - load case id
   @param intflux - array containing internal nodal fluxes and forces
   
   21.3.2004, TKr
*/
void internal_coup_fluxes (double *intflux,long n)
{
  long i,ne,ndofe,*cn;
  elemtypec te;
  vector lif;
  
  ne = Ct->ne;

  for (i=0;i<ne;i++){
    te = Ct->give_elem_type (i);
    ndofe = Gtt->give_ndofe (i);
    allocv (ndofe,lif);
    fillv (0.0,lif);

    cn = new long [ndofe];
    Gtt->give_code_numbers (i,cn);
    
    switch (te){
    case coupbar:{
      Cbar->res_lower_internal_fluxes (i,lif);
      break;
    }
    default:{
      fprintf (stderr,"\n\n unknown element type is required in function");
      fprintf (stderr,"\n internal_coup_fluxes (file %s, line %d).\n",__FILE__,__LINE__);
      abort ();
    }
    }
    
    locglob (intflux,lif.a,cn,ndofe);
    destrv (lif);
    delete [] cn;
    
  }
  //fluxes on boundary
  //bound_fluxes (0,intflux,n);
}



/**
   function assembles %vector of coupling nodal internal forces

   @param lcid - load case id
   @param intflux - array containing internal nodal fluxes
   
*/
void internal_coup_forces (long lcid,double *intflux)
{
  long i,ne,ndofe,*cn;
  elemtypec te;
  vector lif;
  
  ne = Ct->ne;

  for (i=0;i<ne;i++){
    te = Ct->give_elem_type (i);
    ndofe = Gtm->give_ndofe (i);//mefel ndofe
    allocv (ndofe,lif);
    fillv (0.0,lif);

    cn = new long [ndofe];
    Gtm->give_code_numbers (i,cn);//mefel cn
    
    switch (te){
    case coupbar:{
      Cbar->res_upper_internal_forces (i,lif);
      break;
    }
    default:{
      fprintf (stderr,"\n\n unknown element type is required in function");
      fprintf (stderr,"\n internal_coup_forces (file %s, line %d).\n",__FILE__,__LINE__);
      abort ();
    }
    }
    
    locglob (intflux,lif.a,cn,ndofe);
    destrv (lif);
    delete [] cn;
    
  }

  //forces on boundary
  //bound_forces (0,intflux,n);
}


/**
   function assembles coupling right hand side of the problem
   
   @param lcid - load case id
   @param rhs - array containing right hand side
   
   09.03.2004, JK+TKr
*/
void right_hand_side (long lcid,double *rhs,long n)
{
  long i,mndofe,*cn;
  vector lv;
  double *av;
  av = new double [n];
  
  assemble_init_coup_upper (rhs);

  switch (Cp->tmatt){
  case mech_onemedium:{
    nullv (av,n);
    assemble_coup (lcid,av,n);
    addv (rhs,av,n);
    break;
  }
  case mech_twomedia:{
    nullv (av,n);
    assemble_coup (lcid,av,n);
    addv (rhs,av,n);
    break;
  }
  case mech_threemedia:{
    nullv (av,n);
    assemble_coup (lcid,av,n);
    addv (rhs,av,n);
    break;
  }
  default:{
    fprintf (stderr,"\n\n unknown transported matter is required in function");
    fprintf (stderr,"\n right_hand_side (file %s, line %d).\n",__FILE__,__LINE__);
  }
  }
  
  //  contributions from volume integrals
  nullv (av,n);
  for (i=0;i<Ct->ne;i++){
    if (Gtm->leso[i]==1){
      //  only elements switched on are processed
      
      //  number of DOFs on element
      mndofe=Gtm->give_ndofe (i);
      allocv (mndofe,lv);
      fillv (0.0,lv);
      
      //  function is defined in elemswitchc.cpp
      volume_rhs_vectorc (lv,lcid,i);
      
      
      //  code numbers
      cn = new long [mndofe];
      Gtm->give_code_numbers (i,cn);
      
      //  localization of element values to the global vector
      locglob (av,lv.a,cn,mndofe);
      
      destrv (lv);
      delete [] cn;
    }  
  }  
  
  addv (rhs,av,n);
  delete [] av;
}

/**
   function assembles part of the %vector of coupling right hand side
   
   @param rhs - right hand side
   
   21.3.2004, JK+TKr
*/
void assemble_init_coup_upper (double *rhs)
{
  long i,mndofe,tndofe,lcid=0;
  long *rcn,*ccn;
  vector r,f;
  matrix km;
  
  
  for (i=0;i<Ct->ne;i++){
    mndofe = Gtm->give_ndofe (i);
    tndofe = Gtt->give_ndofe (i);
    
    allocv (tndofe,r);
    allocv (mndofe,f);
    allocm (mndofe,tndofe,km);
    
    upper_cond_coupl_mat (i,lcid,km);
    
    rcn = new long [mndofe];
    ccn = new long [tndofe];
    Gtm->give_code_numbers (i,rcn);
    Gtt->give_code_numbers (i,ccn);
    
    elemvalues (0,i,r.a,ccn,tndofe);

    mxv (km,r,f);
    cmulv (-1.0,f);
    
    locglob (rhs,f.a,rcn,mndofe);

    destrm (km);  destrv (f);  destrv (r);
    delete [] rcn;
    delete [] ccn;
  }
}


/**
   function approximates temperature computed in the TRFEL part into the MEFEL part
   
   21.6.2004, JK
*/
void approximation_temper ()
{
  long nnm;
  double *gv;
  
  //  number of nodes in MEFEL
  nnm = Mt->nn;
  
  gv = new double [nnm];
  nullv (gv,nnm);
  
  //  selection or interpolation of temperatures 
  nodal_nodal_values (gv,temperature);
  
  //  approximation of nodal data to the data at integration points
  if (Cp->lbb==quad_lin){
    intpointval2 (gv,temperature);
  }
  if (Cp->lbb==quad_quad || Cp->lbb==lin_lin){
    intpointval (gv,temperature,1.0);
  }
  
  /* 
     fprintf (Outc,"\n\n kontrola hodnot na integracnich bodech \n");
     for (long i=0;i<Mm->tnip;i++){
     fprintf (Outc,"\n point %5ld   %le",i,Mm->tempr[i]);
     }
     fprintf (Outc,"\n");
  */

  delete [] gv;
}

/**
   function approximates humidity computed in the TRFEL part into the MEFEL part
   
   22.6.2004, JK
*/
void approximation_humid ()
{
  long nnm;
  double *gv;
  
  //  number of nodes in MEFEL
  nnm = Mt->nn;
  
  gv = new double [nnm];
  nullv (gv,nnm);
  
  //  selection or interpolation of humidities
  nodal_nodal_values (gv,rel_hum);
  
  //  approximation of nodal data to the data at integration points
  if (Cp->lbb==quad_lin){
    intpointval2 (gv,rel_hum);
  }
  if (Cp->lbb==quad_quad || Cp->lbb==lin_lin){
    intpointval (gv,rel_hum,1.0);
  }
  
  delete [] gv;
}


/**
   function returns nodal values of selected quantity from the TRFEL part
   
   @param gv - array of nodal quantity, the number of components is equal to the number of nodes in MEFEL
   @param nmq - name of the value needed

   24/8/2012, TKr
*/
void nodal_nodal_values (double *gv, nonmechquant nmq)
{
  long i,nnt,cn;
  
  nnt = Tt->nn;

  switch (nmq){
    ////////////////////////////
  case temperature:{
    switch (Cp->tmatt){
    case mech_onemedium:{
      if(Tp->mednam == heat){
	for (i=0;i<nnt;i++){
	  
	  cn=Tt->give_dof (i,0);
	  if (cn>0)  gv[i] = Lsrst->lhsi[cn-1]+Lsrst->lhs[cn-1];
	  if (cn<0)  gv[i] = Tb->lc[0].pv[0-cn-1].getval();
	}
      }
      break;
    }
    case mech_twomedia:{
      for (i=0;i<nnt;i++){
	cn=Tt->give_dof (i,1);
	if (cn>0)  gv[i] = Lsrst->lhsi[cn-1]+Lsrst->lhs[cn-1];
	if (cn<0)  gv[i] = Tb->lc[0].pv[0-cn-1].getval();
      }
      break;
    }
    case mech_threemedia:{
      for (i=0;i<nnt;i++){
	cn=Tt->give_dof (i,2);
	if (cn>0)  gv[i] = Lsrst->lhsi[cn-1]+Lsrst->lhs[cn-1];
	if (cn<0)  gv[i] = Tb->lc[0].pv[0-cn-1].getval();
      }
      break;
    }
    default:{
      print_err("unknown type of coupled problem is required",__FILE__,__LINE__,__func__);
    }
    }
    break;
  }
    ////////////////////////////
  case initial_temperature:{
    switch (Cp->tmatt){
    case mech_onemedium:{
      if(Tp->mednam == heat){
	for (i=0;i<nnt;i++){
	  
	  cn=Tt->give_dof (i,0);
	  if (cn>0)  gv[i] = Lsrst->lhsi[cn-1]+Lsrst->lhs[cn-1];
	  if (cn<0)  gv[i] = Tb->lc[0].pv[0-cn-1].getval();
	}
	//for (i=0;i<nnt;i++){
	//gv[i] = Tt->nodes[i].nodvali[0];
	//}
      }
      break;
    }
    case mech_twomedia:{//it must be corrected 14/03/2011
      if(Tp->mednam == heat_moisture){
	//complete here!!!
      }
      break;
    }
    default:{
      print_err("unknown type of transported media is required",__FILE__,__LINE__,__func__);
    }
    }
    break;
  }
    ////////////////////////////
  case water_pressure:{
    
    for (i=0;i<nnt;i++){  
      cn=Tt->give_dof (i,0);
      if (cn>0)  gv[i] = Lsrst->lhsi[cn-1]+Lsrst->lhs[cn-1];
      if (cn<0)  gv[i] = Tb->lc[0].pv[0-cn-1].getval();
    }
    break;
  }
    ////////////////////////////
  case volume_strain_trans:{
    //approximation volume strains from int. points to nodes
    give_nontransq_nodval (gv,nnt,1);
    break;
  }
    ////////////////////////////
  case volume_tot_strain_trans:{
    //approximation total volume strains from int. points to nodes
    give_nontransq_nodval (gv,nnt,2);
    break;
  }
    ////////////////////////////
  case saturation_degree:{
    //approximation degree of saturation from int. points to nodes
    give_nontransq_nodval (gv,nnt,3);
    break;
  }
    ////////////////////////////
  case suction:{
    //approximation suction from int. points to nodes
    give_nontransq_nodval (gv,nnt,4);
    break;
  }
    ////////////////////////////
  case rel_hum:{
    give_nodal_humid(gv,nnt);
    break;
  }
    ////////////////////////////
  default:{
    print_err("unknown type of nmq is required",__FILE__,__LINE__,__func__);
  }
  }  
}


/**
   function approximates initial temperature computed in the TRFEL part into the MEFEL part
   
   21.11.2007, JK
*/
void approximation_inittemper ()
{
  long nnm;
  double *gv;
  
  //  number of nodes in MEFEL
  nnm = Mt->nn;
  
  gv = new double [nnm];
  nullv (gv,nnm);
  
  //  selection or interpolation of temperatures 
  nodal_nodal_values (gv,initial_temperature);
  
  if (Cp->lbb==quad_quad || Cp->lbb==lin_lin){
    intpointval (gv,initial_temperature,1.0);
  }
  if (Cp->lbb==quad_lin){
    intpointval2 (gv,initial_temperature);
  }
  
  delete [] gv;
}



/**
   function computes temperature strains for MEFEL part
   
   11/05/2010, TKr
*/
void compute_metr_tempstrains()
{
  Cmu->compute_tempstrains();
  //Cml->compute_tempstrains();
}


/**
   function set temperature strains to zero for METR part
   
   14/05/2012, TKr
*/
void null_metr_tempstrains()
{
  Cmu->nulltempstrains ();
  //Cml->nulltempstrains ();
}

/**
   function computes eigen strains for TRFEL part
   
   18/09/2012, TKr
*/
void compute_trfel_eigstrains()
{
  Tm->compute_eigstrains();
  //Cml->compute_eigstrains();
}


/**
   function computes eigen strains for MEFEL part
   
   18/09/2012, TKr
*/
void compute_metr_eigstrains()
{
  Cmu->compute_eigstrains();
  //Cml->compute_eigstrains();
}


/**
   function set eigen strains to zero for METR part
   
   18/09/2012, TKr
*/
void null_metr_eigstrains()
{
  Cmu->nulleigstrains ();
  //Cml->nulleigstrains ();
}



/**
   function set temperature strains to zero for MEFEL part
   
   13/05/2010, TKr
*/
void null_mefel_tempstrains()
{
  Mm->nulltempstrains ();
}


/**
   function set eigen strains to zero for MEFEL part
   
   18/09/2012, TKr
*/
void null_mefel_eigstrains()
{
  Mm->nulleigstrains ();
}


/**
   function exports data from METR into MEFEL
   
   05/06/2012, TKr, not completed, tady pokracovat !!!
*/
void metr_mefel ()
{
}

/**
   function exports data from MEFEL into TRFEL
   
   05/06/2012, TKr
*/
void mefel_trfel ()
{
  long i,nnt;
  double *gv;
  
  //  number of nodes in TRFEL
  nnt = Tt->nn;
  
  gv = new double [nnt];
  nullv (gv,nnt);

  switch (Cp->mednam){//names of transported media
  case mech_heat:
    break;
  case mech_water:{
    
    //  interpolation of nontransport quantities
    for(i=Tm->ntrq;i<Mm->nnmq;i++){//only for mechanical vaues
      //from MEFEL into integration points to TRFEL
      intpointvalc (i,1.0);
    }
    break;
  }
  default:{
    print_err("\n Unknown media name is required in ", __FILE__, __LINE__, __func__);
  }
  }
  
  delete [] gv;
}


/**
   function exports data of TRFEL into MEFEL
   
   05/06/2012, TKr
*/
void trfel_mefel ()
{
  long nnm;
  double *gv;
  
  //  number of nodes in MEFEL
  nnm = Mt->nn;
  
  gv = new double [nnm];

  switch (Cp->mednam){//names of transported media
  case mech_heat:{  
    nullv (gv,nnm);
    //  selection or interpolation of temperature
    nodal_nodal_values (gv,temperature);
    
    if (Cp->lbb==quad_quad || Cp->lbb==lin_lin){
      intpointval (gv,temperature,1.0);
    }
    if (Cp->lbb==quad_lin){
      intpointval2 (gv,temperature);
    }
    break;
  }

  case mech_water:{
    nullv (gv,nnm);
    //  selection or interpolation of capillary water pressure
    nodal_nodal_values (gv,water_pressure);

    if (Cp->lbb==quad_quad || Cp->lbb==lin_lin){
      intpointval (gv,water_pressure,1.0);
    }
    if (Cp->lbb==quad_lin){
      intpointval2 (gv,water_pressure);
    }

    //volume strains from TRFEL part
    nullv (gv,nnm);
    //  selection or interpolation of volume strain caused by changes of pore pressure
    nodal_nodal_values (gv,volume_strain_trans);
    
    if (Cp->lbb==quad_quad || Cp->lbb==lin_lin){
      intpointval (gv,volume_strain_trans,1.0);
    }
    if (Cp->lbb==quad_lin){
      intpointval2 (gv,volume_strain_trans);
    }

    //volume total strains from TRFEL part
    nullv (gv,nnm);
    //  selection or interpolation of volume strain caused by changes of pore pressure
    nodal_nodal_values (gv,volume_tot_strain_trans);
    
    if (Cp->lbb==quad_quad || Cp->lbb==lin_lin){
      intpointval (gv,volume_tot_strain_trans,1.0);
    }
    if (Cp->lbb==quad_lin){
      intpointval2 (gv,volume_tot_strain_trans);
    }


    //saturation degree from TRFEL part
    nullv (gv,nnm);
    //  selection or interpolation of saturation degree
    nodal_nodal_values (gv,saturation_degree);
    
    if (Cp->lbb==quad_quad || Cp->lbb==lin_lin){
      intpointval (gv,saturation_degree,1.0);
    }
    if (Cp->lbb==quad_lin){
      intpointval2 (gv,saturation_degree);
    }

    //suction from TRFEL part
    nullv (gv,nnm);
    //  selection or interpolation of suction pressure
    nodal_nodal_values (gv,suction);
    
    if (Cp->lbb==quad_quad || Cp->lbb==lin_lin){
      intpointval (gv,suction,1.0);
    }
    if (Cp->lbb==quad_lin){
      intpointval2 (gv,suction);
    }

    break;
  }

  case mech_moisture:{
    nullv (gv,nnm);
    //  selection or interpolation of capillary pressure
    nodal_nodal_values (gv,cap_pressure);

    if (Cp->lbb==quad_quad || Cp->lbb==lin_lin){
      intpointval (gv,cap_pressure,1.0);
    }
    if (Cp->lbb==quad_lin){
      intpointval2 (gv,cap_pressure);
    }
    break;
  }

  case mech_heat_moisture:{  
    nullv (gv,nnm);
    //  selection or interpolation of temperature
    nodal_nodal_values (gv,temperature);
    
    if (Cp->lbb==quad_quad || Cp->lbb==lin_lin){
      intpointval (gv,temperature,1.0);
    }
    if (Cp->lbb==quad_lin){
      intpointval2 (gv,temperature);
    }

    nullv (gv,nnm);
    //  selection or interpolation of humidities
    nodal_nodal_values (gv,rel_hum);
    
    //  approximation of nodal data to the data at integration points
    if (Cp->lbb==quad_lin){
      intpointval2 (gv,rel_hum);
    }
    if (Cp->lbb==quad_quad || Cp->lbb==lin_lin){
      intpointval (gv,rel_hum,1.0);
    }

    break;
  }
    
  default:{
    print_err("\n Unknown media name is required in ", __FILE__, __LINE__, __func__);
  }
  }

  delete [] gv;
}


/**
   function exports initial data of TRFEL into MEFEL
   
   05/06/2012, TKr
*/
void init_trfel_mefel ()
{
  long nnm;
  double *gv;
  
  //  number of nodes in MEFEL
  nnm = Mt->nn;
  
  gv = new double [nnm];

  switch (Cp->mednam){//names of transported media
  case mech_heat:{  
    nullv (gv,nnm);
    
    //  selection or interpolation of temperatures 
    nodal_nodal_values (gv,initial_temperature);
    
    if (Cp->lbb==quad_quad || Cp->lbb==lin_lin){
      intpointval (gv,initial_temperature,1.0);
    }
    if (Cp->lbb==quad_lin){
      intpointval2 (gv,initial_temperature);
    }
    break;
  }

  case mech_water:
  case mech_moisture:{
    break;
  }

  case mech_heat_moisture:{  
    nullv (gv,nnm);
    
    //  selection or interpolation of temperatures 
    nodal_nodal_values (gv,initial_temperature);
    
    if (Cp->lbb==quad_quad || Cp->lbb==lin_lin){
      intpointval (gv,initial_temperature,1.0);
    }
    if (Cp->lbb==quad_lin){
      intpointval2 (gv,initial_temperature);
    }

    
    nullv (gv,nnm);
    //  selection or interpolation of humidities
    nodal_nodal_values (gv,rel_hum);
    
    //  approximation of nodal data to the data at integration points
    if (Cp->lbb==quad_lin){
      intpointval2 (gv,rel_hum);
    }
    if (Cp->lbb==quad_quad || Cp->lbb==lin_lin){
      intpointval (gv,rel_hum,1.0);
    }

    break;
  }
    
  default:{
    print_err("\n Unknown media name is required in ", __FILE__, __LINE__, __func__);
  }
  }

  delete [] gv;
}

/***********************************************************************/
/***********************************************************************/
// this below must be revised and corrected

void export_temp ()
{
  long i,cn;
  
  for (i=0;i<Mt->nn;i++){
    cn=Gtt->give_dof (i,2);
    if (cn>0)   Mb->lc[0].pt[i]=Lsrsc->lhs[cn-1];
    if (cn==0)  Mb->lc[0].pt[i]=0.0;
    if (cn<0){
      Mb->lc[0].pt[i]=Tb->lc[0].pv[0-cn-1].getval()-Tb->lc[0].pv[0-cn-1].getipv();
    }
  }
  
  //  funkce zrusena
  //Mm->compute_tempstrains (0);
  
  //  nahradni funkce, je treba ji overit
  Mm->temprstrains (0);
}


/**
   function exports data of MEFEL into TRFEL
   
*/
/* void mefel_trfel ()
   {
   long i,j,k,ii,jj,nipm,nipt,ippm,ippt;
   double dp;
   
   if (Mt->ne != Tt->ne){
   fprintf (stderr,"\n\n the numbers of elements in MEFEL and TRFEL are not equal,");
   fprintf (stderr,"\n another function for data export must be used (file %s, line %d).\n",__FILE__,__LINE__);
   }
   
   for (i=0;i<Mt->ne;i++){
   nipm=Mt->give_nip (i,0,0);
   for (ii=0;ii<Tp->ntm;ii++){
   for (jj=0;jj<Tp->ntm;jj++){
   
   nipt=Tt->give_nip (i,ii,jj);
   if (nipm != nipt/2){
   fprintf (stderr,"\n\n there are different number of integration points on mechanical and transport elements,");
   fprintf (stderr,"\n the number of element is %ld, block %ld, %ld",i,ii,jj);
   break;
   }
   
   ippt = Tt->elements[i].ipp[ii][jj];
   k=ippt+nipt/2;
   
   ippm = Mt->elements[i].ipp[0][0];
   for (j=0;j<nipm;j++){
   dp = Mm->give_dampar (ippm,0,0);
   Tm->damage[ippt]=dp;
   Tm->damage[k]=dp;
   ippm++;
   ippt++;
   k++;
   }
   }
   }
   }
   
   }
*/

/**
   function exports data of TRFEL into MEFEL
   
*/
/* void trfel_mefel ()
   {
   long i,j;
   double *gv;
   
   gv = new double [Tt->nn];
   

   for (i=0;i<Tt->nn;i++){
   j=Gtt->give_dof (i,0);
   if (j>0){
   gv[i]=Lsrst->lhs[j-1];
   }
   if (j<0){
   gv[i]=Tb->lc[0].pv[0-j-1].getval();
   }
   }
   
   intpointval (gv,moist);

   
   for (i=0;i<Tt->nn;i++){
   //j=Gtt->give_dof (i,1);
   j=Gtt->give_dof (i,0);
   if (j>0){
   gv[i]=Lsrst->lhs[j-1];
   }
   if (j<0){
   gv[i]=Tb->lc[0].pv[0-j-1].getval();
   }
   }
   
   //intpointval (gv,temperature,1.0);//tady?????????
   
   delete [] gv;
   }
*/


